package be.jeffcheasey88.peeratcode.parser.java;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.lang.reflect.Modifier;
import java.util.LinkedList;
import java.util.List;

import be.jeffcheasey88.peeratcode.parser.Parser;
import be.jeffcheasey88.peeratcode.parser.Token;
import be.jeffcheasey88.peeratcode.parser.TokenType;
import be.jeffcheasey88.peeratcode.parser.Tokenizer;
import be.jeffcheasey88.peeratcode.parser.state.BuilderStateTree;
import be.jeffcheasey88.peeratcode.parser.state.RedirectStateTree;
import be.jeffcheasey88.peeratcode.parser.state.StateTree;

public class JavaParser  extends Parser<JavaFile> {
	
	public static long time;
	
	static {
		time = System.currentTimeMillis();
	}
	
	public static void main(String[] args) throws Exception{
		File file = new File("C:\\Users\\jeffc\\eclipse-workspace\\peer-at-code-backend\\src\\be\\jeffcheasey88\\peeratcode\\routes\\Result.java");
		BufferedReader reader = new BufferedReader(new FileReader(file));
		
		time = System.currentTimeMillis();
		
		Parser<JavaFile> parser = new JavaParser();
		JavaFile jFile = new JavaFile();
		parser.parse(reader, jFile);

		System.out.println((System.currentTimeMillis()-time)+"ms");
		
		parser.build(new BufferedWriter(new FileWriter(new File("/home/ParserV2.txt"))));
	}
	
	public JavaParser(){
		Tokenizer tokenizer = new Tokenizer();
		
		StateTree<JavaFile> modifier = new StateTree<JavaFile>().then((validator) -> {
			boolean mod = false;
			while(validator.validate(
					(token) -> getModifier(token.getValue()) > 0, 
					(bag, token) -> {
						int m = getModifier(token.getValue());
						Integer current = bag.get();
						bag.set(current == null ? m : current+m);
					})) mod = true;
			return mod;
		});
		modifier.end((a,b) -> a);
		
		StateTree<JavaFile> type = new StateTree<JavaFile>().then((validator) -> 
			validator.validate(
					(token) -> token.getType().equals(TokenType.NAME),
					(bag, token) -> bag.set(token)));
		StateTree<JavaFile> type_generic_begin = type.then((validator) -> validator.validate(
				(token) -> token.getValue().equals("<"),
				(bag, token) -> bag.set(bag.<Token>get().concat(token))));
		StateTree<JavaFile> type_generic_name = type_generic_begin.then((validator) -> 
		validator.validate(
				(token) -> token.getType().equals(TokenType.NAME),
				(bag, token) -> bag.set(token)));
		StateTree<JavaFile> type_generic_split = type_generic_name.then((validator) -> validator.validate(
				(token) -> token.getValue().equals(","),
				(bag, token) -> bag.set(bag.<Token>get().concat(token))));
		StateTree<JavaFile> type_generic_end = type_generic_name.then((validator) -> validator.validate(
				(token) -> token.getValue().equals(">"),
				(bag, token) -> bag.set(bag.<Token>get().concat(token)))).loop();

		type.end((a,b) -> a);
		type_generic_begin.then(type_generic_name);
		type_generic_split.then(type_generic_name);
		type_generic_end.then(type_generic_name);
		type_generic_end.then(type_generic_split);
		type_generic_end.end((a,b) -> a);
		
		StateTree<JavaFile> clazz_ = new StateTree<>();
		StateTree<JavaFile> clazz = new StateTree<>();
		
		StateTree<JavaFile> clazz_base = clazz.then((validator) -> validator.validate((token) -> token.getValue().equals("class")))
			.then(new RedirectStateTree<>(type, (bag) -> "name"));
		clazz_base.then((validator) -> validator.validate((token) -> token.getValue().equals("{")))
			.end((javafile, bag) -> new Class(bag));
		
		StateTree<JavaFile> clazz_mod = clazz_.then(new RedirectStateTree<>(modifier, (bag) -> "modifier"));
		clazz_mod.then(clazz);
		
		StateTree<JavaFile> importState = new StateTree<>();
		importState.then((validator) -> {
			if(validator.validate(
					(token) -> token.getValue().equals("import"),
					(bag, token) -> bag.set(new LinkedList<>()))){
				
				validator.validate(
						(token) -> token.getValue().equals("static"),
						(bag, token) -> bag.set("static", null));
				
				while(validator.validate(
						(token) -> !token.getValue().equals(";"),
						(bag, token) -> bag.<List<Token>>get().add(token))
					);
				
				return validator.validate((token) -> token.getValue().equals(";"));
			}
			return false;
		}).<JavaElement>end((javafile, bag) -> javafile.addImport(bag));

		
		StateTree<JavaFile> main = new StateTree<>();
		
		BuilderStateTree<JavaFile, JavaElement> pack = main.then((validator) -> {	
			if(validator.validate(
					(token) -> token.getValue().equals("package"),
					(bag, token) -> bag.set(new LinkedList<>()))){
				
				while(validator.validate(
						(token) -> !token.getValue().equals(";"),
						(bag, token) -> bag.<List<Token>>get().add(token))
					);
				
				return validator.validate((token) -> token.getValue().equals(";"));
			}
			return false;
		}).<JavaElement>end((javafile, bag) -> javafile.setPackage(bag));
		pack.multiple(importState);
		pack.multiple(clazz_);
		
		
		System.out.println((System.currentTimeMillis()-time)+"ms");
		
		setTokenizer(tokenizer);
		setStateTree(main);
	}
	
	public static int getModifier(String modifier){
		switch(modifier){
			case "public": return Modifier.PUBLIC;
			case "private": return Modifier.PRIVATE;
			case "protected": return Modifier.PROTECTED;
			case "static": return Modifier.STATIC;
			case "final": return Modifier.FINAL;
			case "synchronized": return Modifier.SYNCHRONIZED;
			case "volatile": return Modifier.VOLATILE;
			case "transient": return Modifier.TRANSIENT;
			case "native": return Modifier.NATIVE;
			case "abstract": return Modifier.ABSTRACT;
			case "strictfp": return Modifier.STRICT;
			default: break;
		}
		return 0;
	}
	
}
